<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class Project_member extends CI_Model
{
	function __construct()
	{
		parent::__construct();
		$this->load->library(OTBASE_PATH.'OT_App');
        $this->load->library(OTBASE_PATH.'OT_User');
	}
	
	// Magic function return false
	public function __call($name, $arguments)
    {
       return false;
    }
    
	// 	return projects list query
	public function projects_list_query($select, $options = array())
	{
		$restricted_fields = array('pr.budget_value', 'pr.actual_value', 'pr.budget_type');
		
		$split = explode(',', $select);
		
		foreach($split as $key => $val) {
			$val = trim($val);
			if ($val && in_array($val,$restricted_fields)) {
				$value = explode('.',$val);
				$val = (isset($value[1]))?$value[1]:$val;
				$split[$key] = '-1 AS '.$val;
			}	
		}
		$select = implode(',', $split);
		
		$this->db->select($select, FALSE);
		$this->db->from('project as pr');
		$this->db->join('project_users as pru', 'pr.entity_id=pru.entity_id and pr.project_id=pru.project_id', 'INNER');
		$this->db->join('client as cl', 'pr.entity_id=cl.entity_id and pr.client_id = cl.client_id', 'INNER');

		$where	= array('pr.entity_id' => $this->ot_app->get('entity_id'), 'pru.user_id' => $this->ot_user->get('user_id'));
		if (!empty($options['filters']))
		{
			$where = array_merge($where, $options['filters']);
		}
		$this->db->where($where);
		
		if (isset($options['order_by']) && $options['order_by']!='') {
			$order_by_direction = !empty($options['order_by_direction'])?$options['order_by_direction']:'desc';
			$this->db->order_by($options['order_by'],$order_by_direction);
		}

		$from = !empty($options['from'])?$options['from']:0;
		if (!empty($options['count']))
		{
			$this->db->limit($options['count'], $from);
		}
		
		$this->db->get();
		return $this->db->last_query();
	}

	public function get_project_name($project_id)
	{
		$this->db->select('concat(pr.project_code, pr.project_name) as project_name', FALSE);
		$this->db->from('project as pr');
		$this->db->where(array('pr.entity_id' => $this->ot_app->get('entity_id'), 'pr.project_id' => $project_id));
		$query = $this->db->get();
     	$result = $query->row_array();
     	return (isset($result['project_name'])?$result['project_name']:'');
	}
	// return projects in as array of standard array indexes
	public function get_projects($select = 'pr.project_id, cl.client_name, pr.project_name, pr.project_code,  pr.members_count, pr.total_tasks, pr.over_due_tasks, pr.created_date', $options = array())
	{
		$query	= $this->db->query($this->projects_list_query($select, $options));
		return $query->result_array();
	}
	
	public function client_projects_count($client_ids, $options = array())
    {
    	$this->db->select('client_id, count(*) as projects_count, count(is_active=1) as active_projects');
    	$this->db->from('project');
    	$this->db->where('entity_id', $this->ot_app->get('entity_id'));

		$this->db->where_in('client_id',$client_ids);
    	if (!empty($options['filters']))
		{
			$this->db->where($options['filters']);
		}

		$this->db->group_by('client_id');
		$query = $this->db->get();

		return $query->result_array();
    }
    
    // update project members count
    public function reset_members_count($project_id)
    {
		$this->db->select('count(project_user_id) as members_count');
		$this->db->from('project_users');
		$this->db->where(array('entity_id'=>$this->ot_app->get('entity_id'), 'project_id' => $project_id));
		$query = $this->db->get();

		$result = $query->row();
		
		$this->db->where('project_id', $project_id);
		$this->db->update('project', array('members_count' => $result->members_count));
		
		return true;
    }
	
	public function get_project_task_types($project_ids, $select = '')
	{
		
		if (!is_array($project_ids)) {
			$project_ids = array($project_ids);
		}

		$select = ($select!='')?$select:'pty.estimated_hours, pty.hourly_rate, pty.task_type_id, pty.project_task_type_id, pty.actual_hours';
		$this->db->select($select, false);
		$this->db->from('project_task_types as pty');
		$this->db->join('task_type as ty', 'pty.entity_id=ty.entity_id and pty.task_type_id=ty.task_type_id', 'INNER');
//		$this->db->join('time_sheet AS ts', 'pty.entity_id=ts.entity_id and  pty.project_id = ts.project_id and pty.task_type_id = ts.task_type_id', 'left');
		$this->db->where(array('pty.entity_id' => $this->ot_app->get('entity_id')));
		$this->db->where_in('pty.project_id', $project_ids);
		$this->db->group_by('pty.project_task_type_id');
		$this->db->order_by('ty.sort_order');
		
		$query = $this->db->get();
		return $query->result_array();
	}
}
