<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require_once(APPPATH . '/models/timesheets/mod.timesheets_pmf.php');

class Timesheets_admin extends Timesheets_pmf
{
	public $error	= '';

	function __construct()
	{
		parent::__construct();
		$this->load->library(OTBASE_PATH.'OT_App');
        $this->load->library(OTBASE_PATH.'OT_User');
	}
	
	public function get_tasks($project_id=0, $task_type_id=0, $term='', $limit=10)
	{
		// get closed task status id
		// $result = $this->closed_status();

	    $this->db->select('pt.task_id, pt.task_number, pt.task_priority, pt.name, pt.is_billable,pt.task_type_id');
		$this->db->from('project_task AS pt');

		$where = array(
			'pt.entity_id'		=> $this->ot_app->get('entity_id'),
			'pt.is_parent'		=> 0,
			// 'pt.task_status_id !=' => $result['task_status_id']
			'pt.is_closed !='	=> 1
		);
		if (!empty($project_id))
		{
			$where['pt.project_id'] =  $project_id;
		}
		if (!empty($task_type_id))
		{
			//$where['pt.task_type_id'] =  $task_type_id;
			$this->db->where('(pt.task_type_id = '.$task_type_id.' OR pt.task_type_id = "")');
		}

		preg_match('/^#(\d+)*/', $term, $matches);
		if (count($matches) >= 2)
		{
			$where['pt.task_number'] =  $matches[1];
		}
		else
		{
			$this->db->like('pt.name', $term);
			$this->db->order_by('pt.name');
			if ($limit)
				$this->db->limit(10);
		}
		$this->db->where($where);

		$query = $this->db->get();

		return $query->result_array();
	}
	
	public function get_timesheet_approve_list($select = '', $options = array())
	{
		$select = ($select)?$select:'u.first_name, u.last_name, u.user_image, min(entry_date) as from_date, max(entry_date) as to_date, count(DISTINCT entry_date) AS days, sum(hours) as hours, u.user_id';
		$query = $this->approve_list_query($select, $options);

		$result = $query->result_array();
		return $result;
	}

	public function get_timesheet_approved_list($select = '', $options = array())
	{
		$select	= !empty($select) ? $select : 'u.user_id, u.first_name, u.last_name, u.user_image, min(entry_date) as from_date, max(entry_date) as to_date, count(DISTINCT entry_date) AS days';
		$query	= $this->approve_list_query($select, $options, 'A');

		$result = $query->result_array();
		return $result;
	}

	public function get_time_content_details($user_id, $options=array(), $status='')
	{
		$this->db->select('pr.project_id, pr.project_code, pr.project_name, ty.name as task_type_name, t.activity_name as project_task_name,sum(t.hours) AS hours, ty.task_type_id, pt.task_id, t.approval_status, t.approved_by');
		$this->db->select("DATE_FORMAT(t.approved_date, '%Y-%m-%d') as approved_date", false);
		$this->db->select("DATE_FORMAT(t.entry_date, '%Y-%m-%d') as entry_date", false);
		$this->db->from('time_sheet AS t');
		$this->db->join('project AS pr','t.entity_id=pr.entity_id and t.project_id=pr.project_id','INNER');
		$this->db->join('task_type AS ty','t.entity_id=ty.entity_id and t.task_type_id=ty.task_type_id','INNER');
		$this->db->join('project_task AS pt','t.entity_id=pt.entity_id and t.project_task_id=pt.task_id','LEFT');

		$where	= array('pr.entity_id'=>$this->ot_app->get('entity_id'), 't.user_id'=>$user_id);
		if (!empty($status))
		{
			$where['t.approval_status']	= $status;
		}
		else
		{
			$where['t.approval_status !=']	= '';
		}

		$this->db->where($where);

		if (!empty($options['filters']))
		{
			foreach($options['filters'] as $key=>$value)
			{
				if (is_array($value))
					$this->db->where_in($key,$value);
				else
					$this->db->where($key,$value);
			}
		}	

		$this->db->group_by('entry_date, pr.project_id, t.task_type_id, t.activity_name');

		$query = $this->db->get();
		$result = $query->result_array();
		// echo $this->db->last_query();
		return $result;
	}
	
	public function approve_list_query($select, $options = array(), $approval_status='S')
	{
		$this->db->select($select);
		$this->db->from('time_sheet AS t');
		$this->db->join('project AS pr','t.entity_id = pr.entity_id and t.project_id=pr.project_id','INNER');
		$this->db->join('entity_users AS u','t.entity_id = u.entity_id and t.user_id=u.user_id','INNER');
		
		$this->db->where(array('t.entity_id'=>$this->ot_app->get('entity_id'), 't.approval_status'=>$approval_status));

		if (!empty($options['filters'])) {
			foreach($options['filters'] as $key=>$value)
				if (is_array($value))
					$this->db->where_in($key,$value);
				else
					$this->db->where($key,$value);
		}
		if(!empty($options['from_date']))
			$this->db->where('t.entry_date >=', $options['from_date']);
			
		if(!empty($options['to_date']))
			$this->db->where('t.entry_date <=', $options['to_date']);

		if(!empty($options['group_by']))
		{
			$this->db->join('client AS cl', 't.entity_id = cl.entity_id and pr.client_id = cl.client_id');
			$this->db->group_by('pr.project_id, cl.client_id');
		}	
		else		
			$this->db->group_by('t.user_id');

		$this->db->order_by('from_date, to_date');

		if (!empty($options['limit']))
			$this->db->limit($options['limit']);
		
		return $this->db->get();
	}

	public function get_team_hours($data)
	{
		$from_date	= (!empty($data['from_date'])) ? $data['from_date'] : null;
		$to_date	= (!empty($data['to_date'])) ? $data['to_date'] : null;

		if ($from_date && $to_date)
		{
			return $this->get_team_hours_days($data);
		}
		else
		{
			unset($data['from_date']);
			unset($data['to_date']);
			return $this->get_team_hours_day($data);
		}
	}

	private function get_team_hours_day($data)
	{
		$result = array();
		$date = (!empty($data['date']))?$data['date']:$this->auth->get_timezone_date('Y-m-d');

		$filter_users	= array();
		$filter_projects	= array();
		if(!empty($data['filter_user_id']))
		{
			$filter_users = explode(',', $data['filter_user_id']);
		}

		if(!empty($data['filter_project_id']))
		{
			$filter_projects = explode(',', $data['filter_project_id']);
			$this->db->select('eu.user_id, eu.first_name, eu.last_name, eu.user_image, eu.time_zone_code, eu.is_active', false);
		} else {		
			$users	= $this->get_team_users('DISTINCT eu.user_id, eu.first_name, eu.last_name, eu.user_image, eu.time_zone_code, eu.is_active', $filter_users, $filter_projects, false);
		
			if (!$users)
				return $result;
			$result['users']	= array_key_index($users, 'user_id');
		}

		// get users hours details
		$this->db->select('t.user_id, t.project_id, concat(pr.project_code, pr.project_name) as project_name, tt.name AS task_type, t.time_sheet_id, t.activity_name, t.is_billable, t.hours, t.notes,t.is_progress, td.from_time, t.estimated_hours, t.project_task_id, pt.task_number, pt.task_status_id, eu.user_id, eu.first_name, eu.last_name, eu.user_image, eu.time_zone_code, eu.is_active', false);
		$this->db->from('time_sheet AS t');
		$this->db->join('time_sheet_detail AS td', 't.entity_id=td.entity_id and td.time_sheet_id = t.time_sheet_id AND t.is_progress = 1 AND td.is_progress=1', 'left');
		$this->db->join('entity_users AS eu', 'eu.user_id=t.user_id');
		$this->db->join('project AS pr', 't.entity_id=pr.entity_id and t.project_id = pr.project_id');
		$this->db->join('task_type AS tt', 't.entity_id=tt.entity_id and t.task_type_id = tt.task_type_id');
		$this->db->join('project_task AS pt', 't.entity_id=pt.entity_id and t.project_task_id=pt.task_id', 'left');
		$this->db->where(array('t.entry_date' => $date, 't.entity_id' => $this->ot_app->get('entity_id')));

		if (!empty($data['filter_project_id']))
		{
			$this->db->where_in('t.project_id', $filter_projects);
			if ($filter_users) {
				$this->db->where_in('t.user_id',$filter_users);
			}
			$this->db->order_by('eu.first_name');
		} else {
			$this->db->where_in('t.user_id', array_keys($result['users']));
		}

		$query = $this->db->get();
		$details	= $query->result_array();

		
		foreach ($details as $key=>$val)
		{
			if (!empty($data['filter_project_id']))
			{
				$result['users'][$val['user_id']] = array('user_id' => $val['user_id'], 'first_name' => $val['first_name'], 'last_name' => $val['last_name'], 'user_image' => $val['user_image'], 'time_zone_code' => $val['time_zone_code'], 'is_active' => $val['is_active']);
				unset($val['first_name']);
				unset($val['last_name']);
				unset($val['user_image']);
				unset($val['time_zone_code']);
				unset($val['is_active']);
			}
			$details[$key]['from_time']	= empty($val['from_time']) ? "" : time_in_12_hour_format($val['from_time'], true);
			$details[$key]['cur_time']	= time_in_12_hour_format($this->auth->get_timezone_date('', $result['users'][$val['user_id']]['time_zone_code']), true);
		}
		$result['details'] = $details;
		$result['users']	= isset($result['users'])?$result['users']:array();
		/*if (!empty($data['filter_project_id']))
		{
			$this->db->select('t.user_id, concat(pr.project_code, pr.project_name) as project_name, t.hours', false);
			$this->db->from('time_sheet AS t');
			$this->db->join('project AS pr', 't.entity_id=pr.entity_id and t.project_id = pr.project_id');
			$this->db->where(array('t.entry_date' => $date, 't.entity_id' => $this->ot_app->get('entity_id')));
			$this->db->where_in('t.user_id', array_keys($result['users']));
			$this->db->where_not_in('pr.project_id', $filter_projects);
			$query = $this->db->get();
			$details	= $query->result_array();
			$result['others'] = $details;
		}*/

		return $result;
	}

	public function get_total_hours($options = array())
	{
		
		$this->db->select('sum(hours) AS day_total, is_billable');
		$this->db->select("DATE_FORMAT(entry_date, '%Y-%m-%d') as entry_date", false);
		$this->db->select("invoice_id");
		$this->db->from('time_sheet');
		
		$where	= array(
			'entity_id'	=> $this->ot_app->get('entity_id'),
			'user_id'	=> !empty($options['user_id']) ? $options['user_id'] : $this->ot_user->get('user_id')
		);

		if (!empty($options['filters']))
		{
			$where	= array_merge($where, $options['filters']);
		}
		$this->db->where($where);

		if (!empty($options['where_str']))
		{
			$this->db->where($options['where_str'], NULL, FALSE);
		}

		$group_by = !empty($options['group_by'])?$options['group_by']:'entry_date';	
		$this->db->group_by($group_by);

		$query = $this->db->get();

		return $query->result_array();
	}

	private function get_team_hours_days($data)
	{
		$result				= array();
		$filter_users		= array();
		$filter_projects	= array();

		$date	= (!empty($data['date'])) ? $data['date'] : $this->auth->get_timezone_date('Y-m-d');

		if(!empty($data['filter_user_id']))
		{
			$filter_users	= explode(',', $data['filter_user_id']);
		}

		if(!empty($data['filter_project_id']))
		{
			$filter_projects = explode(',', $data['filter_project_id']);
			$this->db->select('eu.user_id, eu.first_name, eu.last_name, eu.user_image, eu.time_zone_code, eu.is_active', false);
		} else {		
			$users	= $this->get_team_users('DISTINCT eu.user_id, eu.first_name, eu.last_name, eu.user_image, eu.time_zone_code, eu.is_active', $filter_users, $filter_projects, false);
		
			if (!$users)
				return $result;
			$result['users']	= array_key_index($users, 'user_id');
		}

		// get users hours details
		$this->db->select('sum(t.hours) as hours, t.user_id, t.entry_date', false);
		$this->db->from('time_sheet AS t');
		$this->db->join('entity_users AS eu', 'eu.user_id=t.user_id');
		$this->db->where(array(
			't.entry_date <='	=> $data['to_date'],
			't.entry_date >='	=> $data['from_date'],
			't.entity_id'		=> $this->ot_app->get('entity_id')
		));

		if (empty($data['filter_project_id']))
			$this->db->where_in('t.user_id', array_keys($result['users']));

		if (!empty($filter_projects))
		{
			$this->db->where_in('t.project_id', $filter_projects);
			if ($filter_users) {
				$this->db->where_in('t.user_id',$filter_users);
			}
		}

		$this->db->group_by('t.user_id, t.entry_date');

		$query		= $this->db->get();

		$details	= $query->result_array();

		foreach ($details as $key=>$val)
		{
			if (!empty($data['filter_project_id']))
			{
				$result['users'][$val['user_id']] = array('user_id' => $val['user_id'], 'first_name' => $val['first_name'], 'last_name' => $val['last_name'], 'user_image' => $val['user_image'], 'time_zone_code' => $val['time_zone_code'], 'is_active' => $val['is_active']);
				unset($val['first_name']);
				unset($val['last_name']);
				unset($val['user_image']);
				unset($val['time_zone_code']);
				unset($val['is_active']);
			}
			$details[$key]['cur_time']	= time_in_12_hour_format($this->auth->get_timezone_date('', $result['users'][$val['user_id']]['time_zone_code']), true);
		}

		$result['users']	= array_values($result['users']);
		$result['details']	= $details;

		return $result;
	}

	function get_team_users($select = '', $filter_users = array(), $filter_projects=array(), $isActive=true) 
	{
		// get related users
		$select = !empty($select)?$select:'DISTINCT eu.user_id, eu.first_name, eu.last_name, eu.user_image';
		$this->db->select($select, false);
		$this->db->from('entity_users AS eu');
		if(!empty($filter_projects))
			$this->db->join('project_users AS pu', 'pu.user_id = eu.user_id');
		$this->db->where('eu.entity_id', $this->ot_app->get('entity_id'));

		if ($isActive)
		{
			$this->db->where('eu.is_active', 1);
		}

		if(!empty($filter_users))
			$this->db->where_in('eu.user_id',$filter_users);

		if(!empty($filter_projects))
			$this->db->where_in('pu.project_id',$filter_projects);

		$this->db->order_by('eu.first_name');
		$query = $this->db->get();
		$users = $query->result_array();
		
		return $users;
	}

	public function delete_details ($time_sheet_id, $ids = array(), $force = false)
	{
		// delete time sheet details
		if ($ids)
		{
			$this->db->where_in('time_sheet_detail_id', $ids);
		}
		$where = array('entity_id' => $this->ot_app->get('entity_id'), 'time_sheet_id' => $time_sheet_id);
		if (!$force) $where['is_progress'] = 0;
		$this->db->where($where);
		$this->db->delete('time_sheet_detail');

		return ($ids)?$ids:true;
	}
}
/* End of file */
