<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require_once(APPPATH . '/models/users/mod.users_member.php');

class Users_pm extends Users_member {

	function __construct()
	{
		parent::__construct();
	}

	public function get_user_roles()
	{
		$roles	= $this->config->config['user_role'];
		unset($roles['A'], $roles['F'], $roles['P']);
		return $roles;
	}
	
	public function users_list_query($select, $options = array())
	{
		$this->db->select($select);

		$this->db->from('entity_users AS eu');

		$this->db->join('time_sheet AS t', 't.user_id=eu.user_id', 'left');

		$where	= array('eu.entity_id' => $this->ot_app->get('entity_id'));

		if (!empty($options['filters']))
		{
			foreach($options['filters'] as $key=>$val)
			{
				if (is_array($val))
					$this->db->where_in($key,$val);
				else
					$where[$key] = $val;
			}
		}

		$this->db->where($where);

		$order_by	= !isset($options['order_by']) ? 'eu.first_name,eu.last_name' : $options['order_by'];
		if ($order_by)
			$this->db->order_by($order_by);

		$this->db->group_by('eu.user_id');

		$from	= empty($options['from']) ? 0 : $options['from'];
		if (!empty($options['count']))
		{
			$this->db->limit($options['count'], $from);
		}

		return $this->db->get();
	}

	public function get_users($select='', $options=array())
	{
		$select	= empty($select) ? 'eu.user_id, eu.first_name, eu.last_name, eu.email, eu.user_image, eu.department, eu.user_role, eu.time_zone_code, eu.is_active, sum(t.hours) AS hours' : $select;

		$query	= $this->users_list_query($select, $options);
		$return = $query->result_array();
		
		if ($return && strpos($select,'user_role')!==false)
		{
			foreach ($return as $key => $row)
			{
				$row['is_edit_permission'] = false;
				if (is_member($row['user_role']))
				{
					$row['is_edit_permission'] = true;
				}
				$return[$key]	= $row;
			}
		}

		return $return;
	}

	public function get_user($user_id = 0)
	{
		$user_id = intval($user_id);

		if ($user_id <= 0)
		{
			$this->error	= 'INVALID_REQUEST';
			return false;
		}
		
		$roles = array_keys($this->get_user_roles());
		$this->db->where_in('user_role', $roles);
		$this->db->select('user_id, first_name, last_name, email, user_image, department, time_zone_code, is_active');
		$this->db->from('entity_users');
		$this->db->where(array('entity_id' => $this->ot_app->get('entity_id'), 'user_id' => $user_id));

		$query = $this->db->get();

		$data = $query->row_array();

		return $data;
	}
	
	public function have_add_permission()
	{
		return true;
	}

	public function activate($user_id, $active=true)
	{
		$user_id = intval($user_id);
	
		if ($user_id <= 0)
		{
			$this->error	= 'INVALID_REQUEST';
			return false;
		}

		if ($active && !license_limit($user_id, 'users')) {
			$this->response->error(sprintf(lang('UPGRADE_YOUR_VERSION'),'users'));
			return false;
		}
	
		$active = ($active) ? 1 : 0;
		$this->db->where(array('entity_id' => $this->ot_app->get('entity_id'), 'user_id' => $user_id));
		$this->db->update('entity_users', array('is_active'=>$active));
	
		return true;
	}

	public function delete($user_id)
	{
		$user_id = intval($user_id);

		if ($user_id <= 0 && $user_id==$this->ot_user->get('user_id'))
		{
			return false;
		}

		// delete user detail
		$roles = array_keys($this->get_user_roles());
		$this->db->where_in('user_role', $roles);
		$this->db->where(array('entity_id' => $this->ot_app->get('entity_id'), 'user_id' => $user_id, 'created_by' => $this->ot_user->get('user_id')));
		if ($this->db->delete('entity_users')) {
			//delete user activity detail
			$this->db->where(array('entity_id' => $this->ot_app->get('entity_id'), 'user_id' => $user_id));
			$this->db->delete('user_activity');
		}
		
		return true;
	}
	
	public function get_user_count()
	{
		$this->db->select('SUM(is_active=1) AS active_count,SUM(is_active=0) AS archive_count', false);
		$this->db->from('entity_users');
		$this->db->where(array('entity_id' => $this->ot_app->get('entity_id')));
		$query = $this->db->get();

		return $query->row_array();
	}
	
	public function get_projects($user_id = 0, $options = array())
	{
		if ($user_id <= 0)
		{
			$this->error	= 'INVALID_REQUEST';
			return false;
		}

		$this->load->model('projects');
		$projects = $this->projects->get_projects('pr.project_id, pr.project_name');
		$pro_ids = array();
		if ($projects) {
			foreach ($projects as $item) {
				if (!is_member($item['user_role'])) {
					$pro_ids[] = $item['project_id'];
				}
			}
		}
		if (!$pro_ids) return array();
		
		$this->db->select('pr.project_id, concat(pr.project_code, pr.project_name) AS project_name, pr.is_active, pr.created_date, pr.last_updated_date, concat(cl.client_code, cl.client_name) AS client_name, sum(ts.hours) as hours, max(ts.last_updated_date) as date', false);
		$this->db->from('project as pr');
		$this->db->join('project_users as pru', 'pr.entity_id=pru.entity_id and pr.project_id=pru.project_id', 'INNER');
		$this->db->join('client as cl', 'pr.entity_id=cl.entity_id and pr.client_id=cl.client_id', 'INNER');
		$this->db->join('time_sheet as ts', 'pr.entity_id=ts.entity_id and pr.project_id=ts.project_id AND ts.user_id = '.$user_id.'', 'LEFT OUTER');
		$this->db->where(array('pr.entity_id' => $this->ot_app->get('entity_id'), 'pru.user_id' => $user_id));
		$this->db->where_in('pr.project_id', $pro_ids);
		$this->db->group_by('pr.project_id');
		$this->db->order_by('date', 'desc');
	
		$from	= empty($options['from']) ? 0 : $options['from'];
		if (!empty($options['count']))
		{
			$this->db->limit($options['count'], $from);
		}
	
		$query = $this->db->get();
	
		$data = $query->result_array();
	
		return $data;
	}
}
